/*-----------------------------------------------------------------------------
 *   Diffpicture 2 - calculates diffrence between pictures
 *   Copyright (c) 2005 Arne Bochem <diffpicture2@arneb.de>
 * 
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *----------------------------------------------------------------------------*/

#include "stdlib.h"
#include "stdio.h"
#include <IL/il.h>
#include <IL/ilu.h>
#include <IL/ilut.h>
#include <string.h>
#include <math.h>

float calc_difference (unsigned char i1[], unsigned char i2[], int w, int h)
{
	int x, y;
	long long int total = 0;

	w *= 3;

	for (y = 0; y < h; y++)
	{
		for (x = 0; x < w; x += 3)
		{
			total += abs(i1[y * w + x] - i2[y * w + x]) + abs(i1[y * w + x + 1] - i2[y * w + x + 1]) + abs(i1[y * w + x + 2] - i2[y * w + x + 2]);
		}
	}

	return( ((float)total / (float)(w * h))/255.0 );
}

int main (int argc, char *argv[])
{
	ILuint image1, image2;
	unsigned char *i1, *i2;
	int w, h, w2, h2;

	ilInit();
	iluInit();

	if (argc != 3)
	{
		fprintf(stderr, "Invalid number of arguments, usage: diffpicture2 IMAGE IMAGE\n");
		return(1);
	}

	image1 = iluGenImage();
	ilBindImage(image1);
	if ((ilLoadImage(argv[1]) == IL_FALSE) || (ilGetError() != IL_NO_ERROR))
	{
		fprintf(stderr, "Cannot load first image.\n");
		return(1);
	}
	ilConvertImage(IL_RGB, IL_UNSIGNED_BYTE);
	w = ilGetInteger(IL_IMAGE_WIDTH);
	h = ilGetInteger(IL_IMAGE_HEIGHT);
	i1 = ilGetData();

	image2 = iluGenImage();
	ilBindImage(image2);
	if ((ilLoadImage(argv[2]) == IL_FALSE) || (ilGetError() != IL_NO_ERROR))
	{
		fprintf(stderr, "Cannot load second image.\n");
		return(1);
	}
	ilConvertImage(IL_RGB, IL_UNSIGNED_BYTE);
	w2 = ilGetInteger(IL_IMAGE_WIDTH);
	h2 = ilGetInteger(IL_IMAGE_HEIGHT);
	i2 = ilGetData();

	if ((w != w2) || (h != h2))
	{
		fprintf(stderr, "Images are not of the same size.\n");
		return(1);
	}

	printf("%.16f\n", calc_difference(i1, i2, w, h));

	return(0);
}
